@@ -1,30 +1,41 @@
 from __future__ import unicode_literals
-import logging
-logging.debug('Loading utils.py')
-
-import re
 import os
 import sys
 import errno
 import csv
 import codecs
-import io
-from collections import defaultdict
 
-import numpy as np
+try:
+    from cStringIO import StringIO
+except:
+    from io import StringIO
+
+try:
+    import xml.etree.cElementTree as et
+except ImportError:
+    import xml.etree.ElementTree as et
 
 rdbu9 = [0, '#b2182b', '#d6604d', '#f4a582', '#fddbc7', '#cccccc', '#d1e5f0', '#92c5de', '#4393c3', '#2166ac']
 rdbu9c = [0, '#ffffff', '#000000', '#000000', '#000000', '#000000', '#000000', '#000000', '#ffffff', '#ffffff']
 category10 = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
 
 
+def luminahex(hex):
+    hex = hex.strip('#')
+    R, G, B = int(hex[0:2], 16) / 255., int(hex[2:4], 16) / 255., int(hex[4:6], 16) / 255.
+    return lumina(R, G, B)
+
+
+def lumina(R, G, B):
+    return (R + R + B + G + G + G) / 6
+
 
 def _convert_list_type_from_XML(vs):
     '''
     Lists are a complex type with possibility for mixed sub-types. Therefore each
     sub-entity must be wrapped with a type specifier.
     '''
-    vlist = vs.findall('ListItem') + vs.findall('ConfigListItem') # ConfigListItem is legacy
+    vlist = vs.findall('ListItem') + vs.findall('ConfigListItem')  # ConfigListItem is legacy
     l = []
     for xconfig in vlist:
         v = xconfig.text
@@ -48,6 +59,36 @@ def _convert_list_type_to_XML(co, vs):
     return co
 
 
+def _convert_dict_type_from_XML(vs):
+    '''
+    Dicts are a complex type with possibility for mixed sub-types. Therefore each
+    sub-entity must be wrapped with a type specifier.
+    '''
+    vlist = vs.findall('DictItem')
+    d = {}
+    for xconfig in vlist:
+        v = xconfig.text
+        if xconfig.get('type') in CONVERT_TYPE_FROM_XML:
+            # Recursive; woo!
+            v = CONVERT_TYPE_FROM_XML[xconfig.get('type')](xconfig)
+        d[xconfig.get('key')] = v
+    return d
+
+
+def _convert_dict_type_to_XML(co, vs):
+    '''
+    Dicts are a complex type with possibility for mixed sub-types. Therefore each
+    sub-entity must be wrapped with a type specifier.
+    '''
+    for k, v in vs.items():
+        c = et.SubElement(co, "DictItem")
+        t = type(v).__name__
+        c.set("type", t)
+        c.set("key", k)
+        c = CONVERT_TYPE_TO_XML[t](c, v)
+    return co
+
+
 def _apply_text_str(co, s):
     co.text = str(s)
     return co
@@ -58,7 +99,9 @@ CONVERT_TYPE_TO_XML = {
     'int': _apply_text_str,
     'float': _apply_text_str,
     'bool': _apply_text_str,
-    'list': _convert_list_type_to_XML
+    'list': _convert_list_type_to_XML,
+    'tuple': _convert_list_type_to_XML,
+    'dict': _convert_dict_type_to_XML,
 }
 
 CONVERT_TYPE_FROM_XML = {
@@ -66,8 +109,10 @@ CONVERT_TYPE_FROM_XML = {
     'unicode': lambda x: str(x.text),
     'int': lambda x: int(x.text),
     'float': lambda x: float(x.text),
-    'bool': lambda x: bool(x.text),
-    'list': _convert_list_type_from_XML
+    'bool': lambda x: bool(x.text.lower() == 'true'),
+    'list': _convert_list_type_from_XML,
+    'tuple': _convert_list_type_from_XML,
+    'dict': _convert_dict_type_from_XML,
 }
 
 
@@ -87,40 +132,6 @@ def sigstars(p):
     return s
 
 
-def calculate_scale(range, output, out=float):
-# Calculate a scale using equally input points (min,max) or (min, 0, max)
-# mapped to the output range; returns a lambda function to perform the conversion
-
-    if len(range) == 3:  # Have midpoint
-        intersect_in = range[1]
-    else:
-        intersect_in = range[0]
-
-    maxi = max([abs(x) for x in range])
-    mini = -maxi
-
-    # Calculate slope in = x, out = y
-    # slope = y2-y1/x2-x1
-    m = (output[1] - output[0]) / (maxi - mini)
-    x1 = range[0]
-    y1 = output[0]
-    mino = min(output)
-    maxo = max(output)
-
-    scale = lambda x: np.clip(out((m * x) - (m * x1) + y1), mino, maxo)
-    print("Scale generator...")
-    return scale
-
-
-def calculate_rdbu9_color(scale, value):
-    # Rescale minima-maxima to range of rdbu9 (9)
-    try:
-        rdbu9col = int(scale(value))
-    except:
-        return None  # Fill zero nothing if not known
-    return (rdbu9[rdbu9col], rdbu9c[rdbu9col], rdbu9col)
-
-
 def invert_direction(direction):
     if direction == 'forward':
         return 'back'
@@ -139,7 +150,6 @@ def nonull(stream):
         yield line.replace('\x00', '')
 
 
-
 if sys.version_info < (3, 0):  # Python 2 only
 
 
@@ -150,6 +160,7 @@ if sys.version_info < (3, 0):  # Python 2 only
         """
 
         def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwargs):
+
             self.reader = csv.reader(f, **kwargs)
             self.encoding = encoding
             #f = UTF8Recoder(f, encoding)
@@ -175,12 +186,13 @@ if sys.version_info < (3, 0):  # Python 2 only
 
         def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwargs):
             # Redirect output to a queue
-            self.queue = io.StringIO()
+            self.queue = StringIO()
             self.writer = csv.writer(self.queue, dialect=dialect, **kwargs)
             self.stream = f
             self.encoder = codecs.getincrementalencoder(encoding)()
 
         def writerow(self, row):
+            row = [unicode(s) for s in row]
             self.writer.writerow([s.encode("utf-8") for s in row])
             # Fetch UTF-8 output from the queue ...
             data = self.queue.getvalue()
@@ -210,16 +222,18 @@ def mkdir_p(path):
         else:
             raise
 
+
 # http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
 def which(program):
     import os
+
     def is_exe(fpath):
         return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
 
     def ext_candidates(fpath):
-            yield fpath
-            for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
-                yield fpath + ext
+        yield fpath
+        for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
+            yield fpath + ext
 
     fpath, fname = os.path.split(program)
     if fpath:
@@ -231,10 +245,11 @@ def which(program):
             exe_file = os.path.join(path, program)
             for candidate in ext_candidates(exe_file):
                 if is_exe(candidate):
-                    return candidate            
+                    return candidate
 
     return None
 
+
 def find_packager():
 
     import sys
@@ -247,19 +262,29 @@ def find_packager():
         return 'py2exe'
     elif frozen in ('macosx_app', ):
         return 'py2app'
-    elif frozen is True:
+    elif frozen:
         return True  # it doesn't ALWAYS set this return 'cx_Freeze' 
     else:
         return '<unknown packager: %r>' % (frozen, )
 
 # Get current running script folder (Pathomx app folder)
 pkg = find_packager()
-if pkg == None:
+if pkg is None:
     scriptdir = os.path.dirname(os.path.realpath(__file__))  # .rpartition('/')[0]
-elif pkg == True:
+    # Check if Windows NSIST installed (are we running from Program Files
+    if "pkgs" in scriptdir:
+        pkg = 'nsist'
+        basedir = os.path.join(scriptdir, '..', '..')
+    else:
+        # Really not a release package, carry on
+        basedir = os.path.join(scriptdir, '..')
+elif pkg is True:
     scriptdir = os.path.dirname(sys.executable)
+    basedir = scriptdir
 elif pkg == 'py2app':
     #'/Applications/Pathomx.app/Contents/Resources'
     scriptdir = os.environ['RESOURCEPATH']
+    basedir = scriptdir
 elif pkg == 'py2exe':
     scriptdir = os.path.dirname(str(sys.executable, sys.getfilesystemencoding()))
+    basedir = scriptdir
